@@ -34,8 +34,8 @@ GPL.
 // enigmailCommon.js: shared JS functions for Enigmail
 
 // This Enigmail version and compatible Enigmime version
-var gEnigmailVersion = "0.91.0.0";
-var gEnigmimeVersion = "0.91.0.0";
+var gEnigmailVersion = "0.92.0.0";
+var gEnigmimeVersion = "0.92.0.0";
 
 // Maximum size of message directly processed by Enigmail
 const ENIG_MSG_BUFFER_SIZE = 96000;
@@ -182,7 +182,7 @@ var gEnigmailSvc;
 
 function GetEnigmailSvc() {
   // Lazy initialization of enigmail JS component (for efficiency)
-
+  
   if (gEnigmailSvc) {
     return gEnigmailSvc.initialized ? gEnigmailSvc : null;
   }
@@ -208,7 +208,7 @@ function GetEnigmailSvc() {
 
       try {
         // Reset alert count to default value
-        gPrefEnigmail.clearUserPref("initAlertCount");
+        gPrefEnigmail.clearUserPref("initAlert");
       } catch(ex) {
       }
 
@@ -220,7 +220,11 @@ function GetEnigmailSvc() {
 
         errMsg += "\n\n"+EnigGetString("avoidInitErr");
 
-        EnigAlertCount("initAlertCount", "Enigmail: "+errMsg);
+        EnigAlertPref("Enigmail: "+errMsg, "initAlert");
+        if (EnigGetPref("initAlert")) {
+          gEnigmailSvc.initializationAttempted = false;
+          gEnigmailSvc = null;
+        }
       }
 
       return null;
@@ -455,10 +459,17 @@ function EnigReadFileContents(localFile, maxBytes) {
 ///////////////////////////////////////////////////////////////////////////////
 
 function WRITE_LOG(str) {
+  function f00(val, digits) {
+    return ("0000"+val.toString()).substr(-digits);
+  }
+  
+  var d = new Date();
+  var datStr=d.getFullYear()+"-"+f00(d.getMonth()+1, 2)+"-"+f00(d.getDate(),2)+" "+f00(d.getHours(),2)+":"+f00(d.getMinutes(),2)+":"+f00(d.getSeconds(),2)+"."+f00(d.getMilliseconds(),3)+" ";
   if (gEnigLogLevel >= 4)
-    dump(str);
+    dump(datStr+str);
 
   if (gEnigDebugLog && gEnigmailSvc && gEnigmailSvc.logFileStream) {
+    gEnigmailSvc.logFileStream.write(datStr, datStr.length);
     gEnigmailSvc.logFileStream.write(str, str.length);
   }
 }
@@ -1359,46 +1370,6 @@ function EnigDisplayPrefs(showDefault, showPrefs, setPrefs) {
   }
 }
 
-
-function EnigReceiveKey(parent, msgParentWindow, recvFlags, keyId,
-                        progressBar, requestObserver,
-                        errorMsgObj) {
-
-  if (keyId) {
-
-    var valueObj = { keyId: keyId };
-    var checkObj = new Object();
-
-    window.openDialog("chrome://enigmail/content/enigmailKeyserverDlg.xul",
-          "", "dialog,modal,centerscreen", valueObj, checkObj);
-
-    if (! checkObj.value) {
-      errorMsgObj.value = EnigGetString("failCancel");
-      return null;
-    }
-
-    var keyserver = checkObj.value;
-  }
-  else {
-    return null;
-  }
-
-  var enigmailSvc = GetEnigmailSvc();
-  if (!enigmailSvc)
-     return null;
-
-  if (progressBar) {
-    // wait one second before displaying the progress bar
-    var progressParam=ENIG_C.classes["@mozilla.org/messengercompose/composeprogressparameters;1"].createInstance(ENIG_C.interfaces.nsIMsgComposeProgressParams);
-    parent.setTimeout(progressBar.openProgressDialog, 1000, parent, msgParentWindow, "chrome://enigmail/content/enigRetrieveProgress.xul", progressParam);
-    //progressBar.openProgressDialog(parent, msgWindow, "chrome://enigmail/content/enigRetrieveProgress.xul", progressParam);
-    progressBar.onStateChange(null, null, ENIG_C.interfaces.nsIWebProgressListener.STATE_START, 0);
-  }
-
-  return enigmailSvc.receiveKey(recvFlags, keyserver, keyId, requestObserver, errorMsgObj);
-}
-
-
 function EnigFilePicker(title, displayDir, save, defaultExtension, defaultName, filterPairs) {
   DEBUG_LOG("enigmailCommon.js: EnigFilePicker: "+save+"\n");
 
@@ -1470,6 +1441,126 @@ function EnigNewRule(emailAddress) {
   return true;
 }
 
+// Obtain kay list from GnuPG
+function EnigObtainKeyList(secretOnly, refresh) {
+  DEBUG_LOG("enigmailCommon.js: EnigObtainKeyList\n");
+
+  try {
+    var exitCodeObj = new Object();
+    var statusFlagsObj = new Object();
+    var errorMsgObj = new Object();
+
+    var enigmailSvc = GetEnigmailSvc();
+    if (! enigmailSvc)
+      return null;
+    var userList = enigmailSvc.getUserIdList(secretOnly,
+                                             refresh,
+                                             exitCodeObj,
+                                             statusFlagsObj,
+                                             errorMsgObj);
+    if (exitCodeObj.value != 0) {
+      EnigAlert(errorMsgObj.value);
+      return null;
+    }
+  } catch (ex) {
+    ERROR_LOG("ERROR in enigmailUserSelection: enigLoadKeyList\n");
+  }
+
+  if (typeof(userList) == "string") {
+    return userList.split(/\n/);
+  }
+  else {
+    return [];
+  }
+}
+
+// Load the key list into memory
+function EnigLoadKeyList(refresh, keyListObj) {
+  DEBUG_LOG("enigmailCommon.js: EnigLoadKeyList\n");
+
+  var sortUsers = function (a, b) {
+
+   if (a.userId.toLowerCase()<b.userId.toLowerCase()) { return -1;} else {return 1; }
+
+  }
+
+  var aGpgUserList = EnigObtainKeyList(false, refresh);
+  if (!aGpgUserList) return;
+
+  var aGpgSecretsList = EnigObtainKeyList(true, refresh);
+  if (!aGpgSecretsList && !refresh) {
+    if (EnigConfirm(EnigGetString("noSecretKeys"))) {
+      EnigKeygen();
+      EnigLoadKeyList(true, keyListObj);
+    }
+  }
+
+  keyListObj.keyList = [];
+  keyListObj.keySortList = [];
+
+  var keyObj = new Object();
+  var i;
+
+  for (i=0; i<aGpgUserList.length; i++) {
+    var listRow=aGpgUserList[i].split(/:/);
+    if (listRow.length>=0) {
+      switch (listRow[0]) {
+      case "pub":
+        keyObj = new Object();
+        keyObj.expiry=EnigGetDateTime(listRow[EXPIRY], true, false);
+        keyObj.created=EnigGetDateTime(listRow[CREATED], true, false);
+        keyObj.keyId=listRow[KEY_ID];
+        keyObj.keyTrust=listRow[KEY_TRUST];
+        keyObj.keyUseFor=listRow[KEY_USE_FOR];
+        keyObj.ownerTrust=listRow[OWNERTRUST];
+        keyObj.SubUserIds=new Array();
+        keyObj.fpr="";
+        keyObj.photoAvailable=false;
+        keyObj.secretAvailable=false;
+        keyListObj.keyList[listRow[KEY_ID]] = keyObj;
+        break;
+      case "fpr":
+        keyObj.fpr=listRow[USER_ID];
+        break;
+      case "uid":
+        if (typeof(keyObj.userId) != "string") {
+          keyObj.userId=EnigConvertGpgToUnicode(listRow[USER_ID].replace(/\\e3A/g, ":"));
+          keyListObj.keySortList.push({userId: keyObj.userId, keyId: keyObj.keyId});
+        }
+        else {
+          var subUserId = {
+            userId: EnigConvertGpgToUnicode(listRow[USER_ID].replace(/\\e3A/g, ":")),
+            keyTrust: listRow[KEY_TRUST],
+            type: "uid"
+          }
+          keyObj.SubUserIds.push(subUserId);
+        }
+        break;
+      case "uat":
+        if (listRow[USER_ID].indexOf("1 ")==0) {
+          var userId=EnigGetString("userAtt.photo");
+          keyObj.SubUserIds.push({userId: userId, keyTrust:"", type: "uat"});
+          keyObj.photoAvailable=true;
+        }
+      }
+    }
+  }
+
+  // search and mark keys that have secret keys
+  for (i=0; i<aGpgSecretsList.length; i++) {
+     listRow=aGpgSecretsList[i].split(/:/);
+     if (listRow.length>=0) {
+       if (listRow[0] == "sec") {
+         if (typeof(keyListObj.keyList[listRow[KEY_ID]]) == "object") {
+           keyListObj.keyList[listRow[KEY_ID]].secretAvailable=true;
+         }
+       }
+     }
+  }
+
+  keyListObj.keySortList.sort(sortUsers);
+}
+
 
 function EnigEditKeyTrust(userId, keyId) {
   var inputObj = {
@@ -1547,6 +1638,12 @@ function EnigCreateRevokeCert(keyId, userId) {
   return r;
 }
 
+function EngmailCardDetails() {
+  EnigOpenWin("enigmail:cardDetails",
+              "chrome://enigmail/content/enigmailCardDetails.xul",
+              "centerscreen");
+}
+
 
 // return the label of trust for a given trust code
 function EnigGetTrustLabel(trustCode) {
@@ -1588,6 +1685,25 @@ function EnigGetTrustLabel(trustCode) {
   return keyTrust;
 }
 
+function EnigGetDateTime(dateNum, withDate, withTime) {
+  if (dateNum != 0) {
+    var dat=new Date(dateNum * 1000);
+    var appLocale = Components.classes[ENIG_LOCALE_SVC_CONTRACTID].getService(Components.interfaces.nsILocaleService).getApplicationLocale();
+    var dateTimeFormat = Components.classes[ENIG_DATE_FORMAT_CONTRACTID].getService(Components.interfaces.nsIScriptableDateFormat);
+
+    var dateFormat = (withDate ? dateTimeFormat.dateFormatShort : dateTimeFormat.dateFormatNone);
+    var timeFormat = (withTime ? dateTimeFormat.timeFormatNoSeconds : dateTimeFormat.timeFormatNone);
+    return dateTimeFormat.FormatDateTime(appLocale.getCategory("NSILOCALE_TIME"),
+              dateFormat,
+              timeFormat,
+              dat.getFullYear(), dat.getMonth()+1, dat.getDate(),
+              dat.getHours(), dat.getMinutes(), 0);
+  }
+  else {
+    return "";
+  }
+}
+
 function enigCreateInstance (aURL, aInterface) 
 {
   return ENIG_C.classes[aURL].createInstance(ENIG_C.interfaces[aInterface]);
