@@ -1,3 +1,36 @@
+/*
+The contents of this file are subject to the Mozilla Public
+License Version 1.1 (the "MPL"); you may not use this file
+except in compliance with the MPL. You may obtain a copy of
+the MPL at http://www.mozilla.org/MPL/
+
+Software distributed under the MPL is distributed on an "AS
+IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
+implied. See the MPL for the specific language governing
+rights and limitations under the MPL.
+
+The Original Code is Enigmail.
+
+The Initial Developer of the Original Code is Ramalingam Saravanan.
+Portions created by Ramalingam Saravanan <svn@xmlterm.org> are
+Copyright (C) 2001 Ramalingam Saravanan. All Rights Reserved.
+
+Contributor(s):
+Patrick Brunschwig <patrick.brunschwig@gmx.net>
+
+Alternatively, the contents of this file may be used under the
+terms of the GNU General Public License (the "GPL"), in which case
+the provisions of the GPL are applicable instead of
+those above. If you wish to allow use of your version of this
+file only under the terms of the GPL and not to allow
+others to use your version of this file under the MPL, indicate
+your decision by deleting the provisions above and replace them
+with the notice and other provisions required by the GPL.
+If you do not delete the provisions above, a recipient
+may use your version of this file under either the MPL or the
+GPL.
+*/
+
 // Uses: chrome://enigmail/content/enigmailCommon.js
 
 // Initialize enigmailCommon
@@ -16,7 +49,7 @@ const EnigOutputLFLineBreak   = 1024;
 const ENIG_ENIGMSGCOMPFIELDS_CONTRACTID = "@mozdev.org/enigmail/composefields;1";
 
 // List of hash algorithms for PGP/MIME signatures
-var gMimeHashAlgorithms = ["md5", "sha1", "ripemd160"];
+var gMimeHashAlgorithms = ["md5", "sha1", "ripemd160", "sha256", "sha384", "sha512"];
 
 var gEnigEditor;
 var gEnigDirty, gEnigProcessed, gEnigTimeoutID;
@@ -162,6 +195,10 @@ function enigSetSendDefaultOptions() {
   }
 }
 
+function enigRemoveAttachedSig() {
+//  var bucketList = document.getElementById("attachmentBucket");
+//  var node = bucketList.firstChild;
+}
 
 function enigComposeOpen() {
   DEBUG_LOG("enigmailMsgComposeOverlay.js: enigComposeOpen\n");
@@ -174,6 +211,7 @@ function enigComposeOpen() {
           DEBUG_LOG("enigmailMsgComposeOverlay.js: enigComposeOpen: has encrypted originalMsgUri\n");
           DEBUG_LOG("originalMsgURI="+gMsgCompose.originalMsgURI+"\n");
           enigSetSendMode('encrypt');
+          enigRemoveAttachedSig();
         }
       }
       if (typeof(gMsgCompose.compFields.draftId)=="string") {
@@ -273,8 +311,22 @@ function enigTogglePGPMime() {
   gEnigSendPGPMime = !gEnigSendPGPMime;
 }
 
-function enigInsertKey() {
-  DEBUG_LOG("enigmailMsgComposeOverlay.js: enigInsertKey: \n");
+function enigAttachOwnKey() {
+  DEBUG_LOG("enigmailMsgComposeOverlay.js: enigAttachOwnKey:\n");
+  
+  var userIdValue;
+  if (gEnigIdentity.getIntAttribute("pgpKeyMode")>0) {
+    userIdValue = gEnigIdentity.getCharAttribute("pgpkeyId");
+  }
+  else {
+    userIdValue = gEnigIdentity.email;
+  }
+
+  enigExtractAndAttachKey( [userIdValue] );
+}
+
+function enigAttachKey() {
+  DEBUG_LOG("enigmailMsgComposeOverlay.js: enigAttachKey: \n");
 
   var resultObj = new Object();
   var inputObj = new Object();
@@ -285,31 +337,64 @@ function enigInsertKey() {
   window.openDialog("chrome://enigmail/content/enigmailUserSelection.xul","", "dialog,modal,centerscreen", inputObj, resultObj);
   try {
     if (resultObj.cancelled) return;
-    userIdValue = resultObj.userList.join(" ");
+    enigExtractAndAttachKey(resultObj.userList);
   } catch (ex) {
     // cancel pressed -> do nothing
     return;
   }
+}
 
+function enigExtractAndAttachKey(uid) {
+  DEBUG_LOG("enigmailMsgComposeOverlay.js: enigAttachKey: \n");
   var enigmailSvc = GetEnigmailSvc();
   if (!enigmailSvc)
     return;
 
-  var exitCodeObj = new Object();
-  var errorMsgObj = new Object();
+  var tmpDir=EnigGetTempDir();
 
-  var keyBlock = enigmailSvc.extractKey(window, 0, userIdValue, null,
-                                        exitCodeObj, errorMsgObj);
-  var exitCode = exitCodeObj.value;
+  try {
+    var tmpFile = Components.classes[ENIG_LOCAL_FILE_CONTRACTID].createInstance(Components.interfaces.nsILocalFile);
+    tmpFile.initWithPath(tmpDir);
+    if (!(tmpFile.isDirectory() && tmpFile.isWritable())) {
+      EnigAlert(EnigGetString("noTempDir"));
+      return;
+    }
+  }
+  catch (ex) {}
+  tmpFile.append("key.asc");
+  tmpFile.createUnique(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
 
-  if (!keyBlock || (exitCode != 0)) {
-    // Error processing
-    var errorMsg = errorMsgObj.value;
-    EnigAlert(errorMsg);
+  // save file
+  var exitCodeObj= {};
+  var errorMsgObj = {};
+
+  enigmailSvc.extractKey(window, 0, uid.join(" "), tmpFile.path, exitCodeObj, errorMsgObj);
+  if (exitCodeObj.value != 0) {
+    EnigAlert(errorMsgObj.value);
     return;
   }
 
-  EnigEditorInsertText(EnigGetString("pubKey",userIdValue) + keyBlock);
+  // create attachment
+  var ioServ = Components.classes[ENIG_IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
+  var tmpFileURI = ioServ.newFileURI(tmpFile);
+  var keyAttachment = Components.classes["@mozilla.org/messengercompose/attachment;1"].createInstance(Components.interfaces.nsIMsgAttachment);
+  keyAttachment.url = tmpFileURI.spec;
+  if ((uid.length == 1) && (uid[0].search(/^(0x)?[a-fA-F0-9]+$/)==0)) {
+    keyAttachment.name = "0x"+uid[0].substr(-8,8)+".asc";
+  }
+  else {
+    keyAttachment.name = "pgpkeys.asc";
+  }
+  keyAttachment.temporary = true;
+  keyAttachment.contentType = "application/pgp-keys";
+
+  // add attachment to msg
+  AddAttachment(keyAttachment);
+  var attachmentBox = document.getElementById("attachments-box");
+  attachmentBox.hidden = false;
+  document.getElementById("attachmentbucket-sizer").hidden=false;
+
+  gContentChanged = true;
 }
 
 function enigUndoEncryption() {
@@ -1249,18 +1334,18 @@ function enigEncryptMsg(msgSendType) {
           }
 
           // make sure plaintext is not changed to 7bit
-          if (typeof(msgCompFields.forceCharSetEncoding) == "boolean") {
-            msgCompFields.forceCharSetEncoding = true;
-            DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: enabled forceCharSetEncoding\n");
+          if (typeof(msgCompFields.forceMsgEncoding) == "boolean") {
+            msgCompFields.forceMsgEncoding = true;
+            DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: enabled forceMsgEncoding\n");
           }
        }
        catch (ex) {}
     }
     else if ((! usingPGPMime) && (sendFlags & ENIG_ENCRYPT)) {
-      if (typeof(msgCompFields.forceCharSetEncoding) == "boolean") {
+      if (typeof(msgCompFields.forceMsgEncoding) == "boolean") {
         // force keeping the charset (i.e. don't convert to us-ascii)
-        msgCompFields.forceCharSetEncoding = true;
-        DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: enabled forceCharSetEncoding\n");
+        msgCompFields.forceMsgEncoding = true;
+        DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: enabled forceMsgEncoding\n");
       }
     }
 
@@ -1527,14 +1612,8 @@ function enigGenericSendMessage( msgType )
         }
         msgWindow.SetDOMWindow(window);
         
-        if (typeof(getCurrentAccountKey)=="function") {
-          // Mozilla >= 1.7a
-          gMsgCompose.SendMsg(msgType, getCurrentIdentity(), getCurrentAccountKey(), msgWindow, progress);
-        }
-        else {
-          //Mozilla <= 1.6
-          gMsgCompose.SendMsg(msgType, getCurrentIdentity(), msgWindow, progress);
-        }
+        // Mozilla >= 1.7a only
+        gMsgCompose.SendMsg(msgType, getCurrentIdentity(), getCurrentAccountKey(), msgWindow, progress);
         
 /////////////////////////////////////////////////////////////////////////
 // MODIFICATION
