@@ -237,7 +237,6 @@ function enigmailBuildList(refresh) {
          userObj = new Object();
          userObj.expiry=listRow[EXPIRY];
          userObj.created=listRow[CREATED];
-         userObj.userId=EnigConvertGpgToUnicode(listRow[USER_ID].replace(/\\e3A/g, ":"));
          userObj.keyId=listRow[KEY_ID];
          userObj.keyTrust=listRow[KEY_TRUST];
          if (listRow[KEY_USE_FOR].indexOf("D")>=0) {
@@ -250,11 +249,16 @@ function enigmailBuildList(refresh) {
          aUserList.push(userObj);
          break;
        case "uid":
-         var userId = {
-           userId: EnigConvertGpgToUnicode(listRow[USER_ID].replace(/\\e3A/g, ":")),
-           trustLevel: listRow[KEY_TRUST]
-         };
-         userObj.SubUserIds.push(userId);
+         if (typeof(userObj.userId) != "string") {
+           userObj.userId=EnigConvertGpgToUnicode(listRow[USER_ID].replace(/\\e3A/g, ":"));
+         }
+         else {
+           var userId = {
+             userId: EnigConvertGpgToUnicode(listRow[USER_ID].replace(/\\e3A/g, ":")),
+             trustLevel: listRow[KEY_TRUST]
+           };
+           userObj.SubUserIds.push(userId);
+         }
          break;
        case "sub":
          if ((listRow[KEY_USE_FOR].indexOf("e")>=0) &&
@@ -305,8 +309,7 @@ function enigmailBuildList(refresh) {
    toAddr=toAddrList.join(",")+" ";
 
    var d = new Date();
-   // create an ANSI date string (YYYYMMDD) for "now"
-   var now=(d.getDate()+100*(d.getMonth()+1)+10000*(d.getYear()+1900)).toString();
+   var now=d.valueOf() / 1000;
    var aValidUsers = new Array();
 
    var mailAddr, escapedMailAddr;
@@ -321,8 +324,8 @@ function enigmailBuildList(refresh) {
         if (((!aUserList[i].keyTrust) ||
               KEY_NOT_VALID.indexOf(aUserList[i].keyTrust)<0) &&
               aUserList[i].subkeyOK &&
-              ((!aUserList[i].expiry.length) ||
-              (aUserList[i].expiry.length && aUserList[i].expiry.replace(/\-/g, "") >= now))) {
+              ((!aUserList[i].expiry>0) ||
+              (aUserList[i].expiry >= now))) {
             // key still valid
             try {
               mailAddr = EnigStripEmail(aUserList[i].userId).toLowerCase();
@@ -402,7 +405,7 @@ function enigmailBuildList(refresh) {
           var subChildren=document.createElement("treechildren");
           for (user=0; user<aUserList[i].SubUserIds.length; user++) {
             if (KEY_NOT_VALID.indexOf(aUserList[i].SubUserIds[user].trustLevel)<0) {
-              subItem=enigUserSelCreateRow(aUserList[i], -1, aUserList[i].SubUserIds[user].userId, "", "", aUserList[i].SubUserIds[user].trustLevel, true);
+              var subItem=enigUserSelCreateRow(aUserList[i], -1, aUserList[i].SubUserIds[user].userId, "", "", aUserList[i].SubUserIds[user].trustLevel, true);
               subChildren.appendChild(subItem);
             }
           }
@@ -468,7 +471,7 @@ function enigUserSelCreateRow (userObj, activeState, userId, keyValue, dateField
   trustCol.setAttribute("id", "trust");
 
   userCol.setAttribute("label", userId);
-  expCol.setAttribute("label", dateField);
+  expCol.setAttribute("label", EnigGetDateTime(dateField,true, false));
 
   var keyCol=document.createElement("treecell");
   keyCol.setAttribute("label", keyValue.substring(8,16));
